<?php

namespace Drupal\mcapi_mass;

use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionAssembleEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 *
 * Only the comptroller of the system could own this wallet.
 * If the cashier changes - the wallet goes with the new cashier.
 *
 * For the payer wallet:
 * If <grace-period> in weeks is less then how long the wallet has been in the system {
 *   and <percent-of-transaction> is > 0 {
 *   <fee> = <transaction value> * <percent-of-transaction> / 100
 * }
 */
class TransactionSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array {
    return [
      McapiEvents::ASSEMBLE => ['onmakeChildren']
    ];
  }

  /**
   * Charge a fee in the first listed currency.
   *
   * @param TransactionAssembleEvent $event
   *   The event.
   *
   * @note The only performance trial I've done is 511 children with 128MB RAM on a fairly complex site.
   */
  public function onMakeChildren(TransactionAssembleEvent $event) {
    $parent_transaction = $event->getSubject();
    if ($parent_transaction->type->target_id == 'mass') {
      $many = $parent_transaction->many;
      unset($parent_transaction->many);
      $field = $parent_transaction->manyField;
      foreach ($many as $wallet_id) {
        $child_transaction = clone($parent_transaction);
        $child_transaction->{$field}->target_id = $wallet_id;
        if ($child_transaction->payer->target_id == $child_transaction->payee->target_id) {
          continue;
        }
        $event->addChild($child_transaction);
      }
    }
  }

}
