<?php

namespace Drupal\mcapi\Event;

use Drupal\mcapi\Entity\Transaction;
use Symfony\Component\EventDispatcher\GenericEvent;

/**
 * Event which is fired before a new transaction is validated.
 *
 * @see \Drupal\mcapi\Entity\Transaction::assemble()
 */
class TransactionAssembleEvent extends GenericEvent {

  const EVENT_NAME = 'mcapi_transaction.assemble';

  public $children = [];

  /**
   * Add a child transaction.
   *
   * @param array $props
   *   The new child transaction.
   */
  public function addChildTransaction(array $props) {
    $parent_transaction = $this->getTransaction();
    if ($props['payer'] == $props['payee']) {
      \Drupal::logger('Community Accounting')->alert(
        'Skipped auto transaction with same payer and payee: @name',
        ['@name' => $props['payee']]
      );
      return;
    }
    // References \Drupal\mcapi\Entity\Type.
    $props['type'] = $props['type']??'auto';
    $props['state'] = $parent_transaction->state->target_id;
    $child_transaction = Transaction::create($props);
    // check the value wasn't rounded down to nothing
    if (!$child_transaction->worth->isEmpty()) {
      $this->children[] = $child_transaction;
    }
  }

  public function addChild(Transaction $transaction) {
    $parent_transaction = $this->getTransaction();
    $transaction->uuid->applyDefaultValue(FALSE);
    $transaction->type->target_id = $transaction->type->target_id??'auto';
    $transaction->state->target_id = $parent_transaction->state->target_id;
    $this->children[] = $transaction;
  }

  /**
   * Get the parent transaction.
   *
   * @return Transaction
   */
  public function getTransaction() {
    return $this->getSubject();
  }


}
